"use strict";

const path = require('path');
const fs = require('fs');
const {
  getAllI18n,
  getPropertiesAsJSON,
  getAllI18nGroupedBySection
} = require('../I18nSplitPlugin/utils/propertiesUtils');
const {
  collectI18nKeysfromAST,
  collectI18nKeysfromComments
} = require('../I18nSplitPlugin/utils/collectI18nKeys');
const NullFactory = require('webpack/lib/NullFactory');
const {
  I18nDependency,
  I18nDependencyTemplate
} = require('../I18nSplitPlugin/I18nDependency');
const webpack = require('webpack');
const {
  RawSource
} = require('webpack-sources'); // Import RawSource from webpack-sources

const pluginName = 'I18nPlugin';
class I18nPlugin {
  constructor(options = {}) {
    const {
      jsResourcePath,
      mainChunkName,
      propertiesFolder,
      disableDefault,
      jsResource
    } = options;
    if (!jsResourcePath) {
      throw new Error('I18nPlugin: "jsResourcePath" option is required.');
    }
    if (!mainChunkName) {
      throw new Error('I18nPlugin: "mainChunkName" option is required.');
    }
    this.jsResourcePath = jsResourcePath;
    this.entryPointName = mainChunkName;
    this.jsResourceI18nKeys = jsResource ? (0, getPropertiesAsJSON)(jsResource) : {};
    this.allI18nKeys = (0, getAllI18n)({
      folderPath: propertiesFolder,
      disableDefault,
      jsResourceI18nKeys: this.jsResourceI18nKeys
    });
    console.log("allI18nKeys count:", Object.keys(this.allI18nKeys).length);
    this.moduleToKeysMap = new Map();
    this.keyToModulesMap = new Map();
    this.keyToI18nChunkIdMap = {};
    this.sectionedI18nKeys = getAllI18nGroupedBySection({
      folderPath: propertiesFolder,
      disableDefault,
      jsResourceI18nKeys: this.jsResourceI18nKeys
    });
  }
  defineI18nDependency(compilation) {
    compilation.dependencyFactories.set(I18nDependency, new NullFactory());
    compilation.dependencyTemplates.set(I18nDependency, new I18nDependencyTemplate());
  }
  addI18nDependency(module, i18nKeys) {
    const identifier = `i18n ${module.resource}`;
    const dep = new I18nDependency({
      identifier,
      i18nKeys
    }, module.context, 0);
    module.addDependency(dep);
  }
  apply(compiler) {
    compiler.hooks.thisCompilation.tap(pluginName, (compilation, params) => {
      this.defineI18nDependency(compilation); // handler for parser

      const handler = parser => {
        parser.hooks.program.tap(pluginName, (ast, comments) => {
          const {
            module
          } = parser.state; //

          if (!(module && /\.jsx?$/.test(module.resource))) {
            return;
          } // i18n grep proper contract regex

          let i18nKeys = collectI18nKeysfromAST(ast, this.jsResourceI18nKeys);
          let commentKeys = collectI18nKeysfromComments(comments, this.jsResourceI18nKeys);
          i18nKeys = i18nKeys.concat(commentKeys);
          if (i18nKeys.length) {
            //  console.log(`Collected i18n keys for module ${module.resource}:`, i18nKeys);
            // Store the keys for this module
            this.moduleToKeysMap.set(module.resource, i18nKeys); // Map keys to modules

            for (const key of i18nKeys) {
              if (!this.keyToModulesMap.has(key)) {
                this.keyToModulesMap.set(key, new Set());
              }
              this.keyToModulesMap.get(key).add(module.resource);
            } // Add I18nDependency to the module
          }
        });
      }; // this below  hook was tapped for collect I18n Key from files and add as Deppenensy

      const factory = params.normalModuleFactory;
      factory.hooks.parser.for('javascript/auto').tap(pluginName, handler);
      factory.hooks.parser.for('javascript/dynamic').tap(pluginName, handler);
      compilation.hooks.afterOptimizeChunks.tap(pluginName, () => {
        this.identifyEntryPoints(compilation);
        this.processCollectedKeys(compilation);
      });
    });
  }
  identifyEntryPoints(compilation) {
    this.entryPoints = new Set();
    const entryModules = new Set();
    console.log("Starting identifyEntryPoints..."); // Iterate over each entrypoint in the compilation

    compilation.entrypoints.forEach((entrypoint, name) => {
      console.log(`Processing entrypoint: ${name}`); // Logs the entrypoint name

      if (name === "main") {
        console.log(`Found the "main" entrypoint.`); // Debug log if "main" is found
      } // Process all chunks associated with the entrypoint

      entrypoint.chunks.forEach(chunk => {
        //  console.log(`Processing chunk: ${chunk.name}`);
        chunk.getModules().forEach(module => {
          if (module.resource) {
            entryModules.add(module.resource); // console.log(`Added module: ${module.resource}`);
          }
        });
      });
    });
    this.entryPoints = entryModules; // console.log("Entry Points Identified:", Array.from(this.entryPoints));
  }
  processCollectedKeys(compilation) {
    console.log("Starting processCollectedKeys..."); // Map to store chunkId to i18n keys

    this.chunkIdToKeysMap = {};
    this.moduleChunkIdToModulesMap = new Map(); // Iterate over all chunks in the compilation

    for (const chunk of compilation.chunks) {
      const chunkId = chunk.name || chunk.id; //  console.log(`Processing chunk with ID: ${chunkId}`);
      // Collect all modules in the chunk

      const modules = chunk.getModules(); //   console.log(`Modules in chunk ${chunkId}:`, modules.map(m => m.resource).filter(Boolean));
      // Collect i18n keys used in these modules

      let chunkKeys = new Set();
      let moduleResources = [];
      for (const module of modules) {
        const moduleResource = module.resource;
        if (moduleResource) {
          moduleResources.push(moduleResource);
          const keys = this.moduleToKeysMap.get(moduleResource);
          if (keys) {
            for (const key of keys) {
              chunkKeys.add(key);
            }
          } else {//  console.warn(`Module without resource in chunk ${chunkId}:`, module);
          }
        }
      } // Convert the set to an array

      chunkKeys = Array.from(chunkKeys); // Store in the map if there are keys

      if (chunkKeys.length > 0) {
        this.chunkIdToKeysMap[chunkId] = chunkKeys;
      } // Map chunkId to module resources

      if (moduleResources.length > 0) {
        this.moduleChunkIdToModulesMap.set(chunkId, moduleResources);
      }
    } //  console.log('moduleChunkIdToModulesMap:', this.moduleChunkIdToModulesMap);
    // Emit the i18n chunks

    this.emitI18nChunks(compilation);
  } //   emitI18nChunks(compilation) {
  //     const manifest = {
  //       moduleToI18nChunkMap: {},
  //       locales: {},
  //     };
  //     // Get the list of locales
  //     const locales = Object.keys(this.allI18nKeys);
  //     // Initialize i18nChunkId counter
  //     let i18nChunkIdCounter = 0;
  //     // Map of i18nChunkId to keys
  //     const i18nChunkIdToKeysMap = {};
  //     // For simplicity, assign all keys to chunks based on size
  //     const desiredChunkSize = 100 * 1024; // 100KB
  //     let currentChunkKeys = [];
  //     let currentChunkSize = 0;
  //     // Use the first locale to estimate key sizes
  //     const firstLocale = locales[0];
  //     const allKeys = Object.keys(this.allI18nKeys[firstLocale]);
  //     for (const key of allKeys) {
  //       const value = this.allI18nKeys[firstLocale][key] || '';
  //       const keySize = Buffer.byteLength(key, 'utf8') + Buffer.byteLength(value, 'utf8');
  //       if (currentChunkSize + keySize > desiredChunkSize) {
  //         // Assign current chunk
  //         const i18nChunkId = i18nChunkIdCounter++;
  //         i18nChunkIdToKeysMap[i18nChunkId] = currentChunkKeys;
  //         // Map keys to i18nChunkId
  //         for (const chunkKey of currentChunkKeys) {
  //           this.keyToI18nChunkIdMap[chunkKey] = i18nChunkId;
  //         }
  //         currentChunkKeys = [];
  //         currentChunkSize = 0;
  //       }
  //       currentChunkKeys.push(key);
  //       currentChunkSize += keySize;
  //     }
  //     // Assign the remaining keys in the current chunk
  //     if (currentChunkKeys.length > 0) {
  //       const i18nChunkId = i18nChunkIdCounter++;
  //       i18nChunkIdToKeysMap[i18nChunkId] = currentChunkKeys;
  //       // Map keys to i18nChunkId
  //       for (const chunkKey of currentChunkKeys) {
  //         this.keyToI18nChunkIdMap[chunkKey] = i18nChunkId;
  //       }
  //     }
  //   //  console.log('keyToI18nChunkIdMap:', this.keyToI18nChunkIdMap);
  //     // Now, generate the i18n chunks
  //     for (const locale of locales) {
  //       manifest.locales[locale] = {};
  //       for (const [i18nChunkId, keys] of Object.entries(i18nChunkIdToKeysMap)) {
  //         const i18nData = {};
  //         for (const key of keys) {
  //           const value = this.allI18nKeys[locale][key];
  //           if (value !== undefined) {
  //             i18nData[key] = value;
  //           }
  //         }
  //         // Skip if no keys are available for this locale
  //         if (Object.keys(i18nData).length === 0) {
  //           console.log(`Locale ${locale}, i18n chunk ${i18nChunkId} has no keys, skipping.`);
  //           continue;
  //         }
  //         // Generate the JavaScript code
  //         const moduleSource = `window.i18nData = window.i18nData || {}; window.i18nData[${i18nChunkId}] = ${JSON.stringify(i18nData, null, 2)};`;
  //         // Define the filename and output path
  //         const filename = `${locale}/chunk-${i18nChunkId}.js`;
  //         // Emit the asset
  //         compilation.emitAsset(
  //           filename,
  //           new RawSource(moduleSource)
  //         );
  //       //  console.log(`Emitted i18n chunk: ${filename}`);
  //         // Update the manifest
  //         manifest.locales[locale][i18nChunkId] = filename;
  //       }
  //     }
  //     // Now, create moduleToI18nChunkMap
  //     for (const [moduleChunkId, moduleResources] of this.moduleChunkIdToModulesMap.entries()) {
  //       //  console.log(`Processing module chunk ID: ${moduleChunkId}`);
  //        // console.log(`Module resources:`, moduleResources);
  //       const requiredI18nChunkIds = new Set();
  //       for (const moduleResource of moduleResources) {
  //         const keys = this.moduleToKeysMap.get(moduleResource);
  //         if (keys) {
  //           for (const key of keys) {
  //             const i18nChunkId = this.keyToI18nChunkIdMap[key];
  //             if (i18nChunkId !== undefined) {
  //               requiredI18nChunkIds.add(i18nChunkId);
  //             }
  //           }
  //         }
  //       }
  //       if (requiredI18nChunkIds.size > 0) {
  //         manifest.moduleToI18nChunkMap[moduleChunkId] = Array.from(requiredI18nChunkIds);
  //       //  console.log(`Mapped module chunk ID ${moduleChunkId} to i18n chunks:`, Array.from(requiredI18nChunkIds));
  //       }else{
  //         console.log(`No required i18n chunks for module chunk ID ${moduleChunkId}`);
  //       }
  //     }
  //     // Emit the manifest file
  //     const manifestSource = new RawSource(`window.i18n = window.i18n || {}; window.i18n.manifest = ${JSON.stringify(manifest, null, 2)};`);
  //     const manifestFilename = 'i18n-chunks/manifest.js';
  //     compilation.emitAsset(
  //       manifestFilename,
  //       manifestSource
  //     );
  //     console.log(`Emitted i18n manifest: ${manifestFilename}`);
  //   }

  emitI18nChunks(compilation) {
    const manifest = {
      moduleToI18nChunkMap: {},
      locales: {}
    }; // Get the list of locales

    const locales = Object.keys(this.allI18nKeys); // Initialize i18nChunkId counter

    let i18nChunkIdCounter = 0;
    const sectionToI18nChunkIdMap = {}; // Assign each section to an i18n chunk
    // Assuming all locales have the same sections

    const firstLocaleSections = Object.keys(this.allI18nKeys[locales[0]]);
    for (const sectionName of firstLocaleSections) {
      const i18nChunkId = i18nChunkIdCounter++;
      sectionToI18nChunkIdMap[sectionName] = i18nChunkId; // Collect keys for this section

      const keys = Object.keys(this.allI18nKeys[locales[0]][sectionName] || {}); // Map keys to i18nChunkId

      for (const key of keys) {
        this.keyToI18nChunkIdMap[key] = i18nChunkId;
      } // Generate the i18n chunk for each locale

      for (const locale of locales) {
        manifest.locales[locale] = manifest.locales[locale] || {};
        const i18nData = {};
        const localeSectionKeys = this.allI18nKeys[locale][sectionName] || {};
        for (const key of keys) {
          const value = localeSectionKeys[key];
          if (value !== undefined) {
            i18nData[key] = value;
          }
        }
        if (Object.keys(i18nData).length > 0) {
          const moduleSource = `window.i18nData = window.i18nData || {}; window.i18nData[${i18nChunkId}] = ${JSON.stringify(i18nData, null, 2)};`;
          const filename = `${locale}/chunk-${i18nChunkId}.js`;
          compilation.emitAsset(filename, new RawSource(moduleSource));
          manifest.locales[locale][i18nChunkId] = filename;
        }
      }
    } // Now, create moduleToI18nChunkMap

    for (const [moduleChunkId, moduleResources] of Object.entries(this.moduleChunkIdToModulesMap)) {
      const requiredI18nChunkIds = new Set();
      for (const moduleResource of moduleResources) {
        const keys = this.moduleToKeysMap.get(moduleResource);
        if (keys) {
          for (const key of keys) {
            const i18nChunkId = this.keyToI18nChunkIdMap[key];
            if (i18nChunkId !== undefined) {
              requiredI18nChunkIds.add(i18nChunkId);
            }
          }
        }
      }
      if (requiredI18nChunkIds.size > 0) {
        manifest.moduleToI18nChunkMap[moduleChunkId] = Array.from(requiredI18nChunkIds);
      } else {
        // console.log(
        `No required i18n chunks for module chunk ID ${moduleChunkId}`; // );
      }
    } // Emit the manifest file

    const manifestSource = new RawSource(`window.i18n = window.i18n || {}; window.i18n.manifest = ${JSON.stringify(manifest, null, 2)};`);
    const manifestFilename = 'i18n-chunks/manifest.js';
    compilation.emitAsset(manifestFilename, manifestSource);
    console.log(`Emitted i18n manifest: ${manifestFilename}`);
  }
}
I18nPlugin.isChunkHasI18n = function (chunk) {
  for (const module of chunk.modulesIterable) {
    for (const dep of module.dependencies) {
      if (dep instanceof I18nDependency) {
        return true;
      }
    }
  }
  return false;
};
module.exports = I18nPlugin;