"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ChunkManager = void 0;

// ChunkManager.js
class ChunkManager {
  constructor(options) {
    this.maxChunkSize = options.maxChunkSize;
    this.minChunkSize = options.minChunkSize;
    this.chunkGroups = new Map(); // Track related keys
  }
  splitChunk(chunkId, keys, numChunks) {
    // Group related keys first
    const groups = this.groupRelatedKeys(keys);
    const newChunks = []; // Create new chunks

    for (let i = 0; i < numChunks; i++) {
      newChunks.push({
        id: `${chunkId}_${i}`,
        keys: [],
        size: 0
      });
    } // Distribute groups to chunks to maintain key locality

    groups.forEach(group => {
      const targetChunk = this.findBestChunkForGroup(newChunks, group);
      if (targetChunk) {
        targetChunk.keys.push(...group);
        targetChunk.size += group.length;
      }
    }); // Balance remaining individual keys

    const ungroupedKeys = this.getUngroupedKeys(keys, groups);
    this.distributeKeys(ungroupedKeys, newChunks);
    return newChunks;
  }
  groupRelatedKeys(keys) {
    const groups = [];
    const keyPrefix = new Map(); // Group by common prefixes first

    keys.forEach(key => {
      const prefix = key.split('.')[0];
      if (!keyPrefix.has(prefix)) {
        keyPrefix.set(prefix, []);
      }
      keyPrefix.get(prefix).push(key);
    }); // Create groups based on size constraints

    keyPrefix.forEach((prefixKeys, prefix) => {
      if (prefixKeys.length <= this.maxChunkSize) {
        groups.push(prefixKeys);
      } else {
        // Split large prefix groups
        const numSubgroups = Math.ceil(prefixKeys.length / this.maxChunkSize);
        const subgroupSize = Math.ceil(prefixKeys.length / numSubgroups);
        for (let i = 0; i < prefixKeys.length; i += subgroupSize) {
          groups.push(prefixKeys.slice(i, i + subgroupSize));
        }
      }
    });
    return groups;
  }
  findBestChunkForGroup(chunks, group) {
    let bestChunk = null;
    let bestScore = -1;
    chunks.forEach(chunk => {
      const score = this.calculateChunkScore(chunk, group);
      if (score > bestScore) {
        bestScore = score;
        bestChunk = chunk;
      }
    });
    return bestChunk;
  }
  calculateChunkScore(chunk, group) {
    // Score based on:
    // 1. Available space
    // 2. Related keys already in chunk
    // 3. Chunk balance
    const availableSpace = this.maxChunkSize - chunk.size;
    if (availableSpace < group.length) return -1;
    let score = 100 * (availableSpace / this.maxChunkSize); // Check for related keys

    const relatedKeys = this.countRelatedKeys(chunk.keys, group);
    score += relatedKeys * 10;
    return score;
  }
  countRelatedKeys(existingKeys, newKeys) {
    let count = 0;
    const prefixes = new Set(newKeys.map(key => key.split('.')[0]));
    existingKeys.forEach(key => {
      if (prefixes.has(key.split('.')[0])) count++;
    });
    return count;
  }
  distributeKeys(keys, chunks) {
    // Sort chunks by available space
    chunks.sort((a, b) => this.maxChunkSize - a.size - (this.maxChunkSize - b.size));
    keys.forEach(key => {
      const targetChunk = chunks.find(chunk => chunk.size < this.maxChunkSize);
      if (targetChunk) {
        targetChunk.keys.push(key);
        targetChunk.size++;
      }
    });
  }
  mergeChunks(chunk1Id, chunk2Id) {
    const chunk1 = this.chunkGroups.get(chunk1Id);
    const chunk2 = this.chunkGroups.get(chunk2Id);
    if (!chunk1 || !chunk2) return null;
    const combinedSize = chunk1.size + chunk2.size;
    if (combinedSize > this.maxChunkSize) return null;
    const mergedChunk = {
      id: `${chunk1Id}_${chunk2Id}_merged`,
      keys: [...chunk1.keys, ...chunk2.keys],
      size: combinedSize
    };
    this.chunkGroups.delete(chunk1Id);
    this.chunkGroups.delete(chunk2Id);
    this.chunkGroups.set(mergedChunk.id, mergedChunk);
    return mergedChunk;
  }
  getUngroupedKeys(allKeys, groups) {
    const groupedKeys = new Set(groups.flat());
    return allKeys.filter(key => !groupedKeys.has(key));
  }
}
exports.ChunkManager = ChunkManager;