"use strict";

// I18nPropertiesPlugin.js
const fs = require('fs');
const path = require('path');
class I18nPropertiesPlugin {
  constructor(options = {}) {
    // Default options
    this.options = Object.assign({
      propertiesDir: '',
      // Directory containing .properties files
      outputDir: 'i18n',
      // Output directory in the build output
      outputFormat: 'json',
      // Output format: 'json' or 'js'
      encoding: 'utf8' // File encoding
    }, options);
  }
  apply(compiler) {
    compiler.hooks.thisCompilation.tap('I18nPropertiesPlugin', compilation => {
      compilation.hooks.processAssets.tapAsync({
        name: 'I18nPropertiesPlugin',
        stage: compilation.constructor.PROCESS_ASSETS_STAGE_ADDITIONS
      }, (assets, callback) => {
        this.processPropertiesFiles(compilation).then(() => callback()).catch(error => callback(error));
      });
    });
  }
  async processPropertiesFiles(compilation) {
    const propertiesDir = path.resolve(this.options.propertiesDir);
    const files = await this.getPropertiesFiles(propertiesDir);
    for (const file of files) {
      const content = fs.readFileSync(file, this.options.encoding);
      const transformed = this.transformProperties(content);
      const relativePath = path.relative(propertiesDir, file);
      const outputFileName = this.getOutputFileName(relativePath);
      const outputContent = this.getOutputContent(transformed); // Emit the file into the Webpack output

      compilation.emitAsset(outputFileName, {
        source: () => outputContent,
        size: () => outputContent.length
      });
    }
  }
  async getPropertiesFiles(dir) {
    const entries = fs.readdirSync(dir, {
      withFileTypes: true
    });
    const files = [];
    for (const entry of entries) {
      const fullPath = path.join(dir, entry.name);
      if (entry.isDirectory()) {
        files.push(...(await this.getPropertiesFiles(fullPath)));
      } else if (entry.isFile() && entry.name.endsWith('.properties')) {
        files.push(fullPath);
      }
    }
    return files;
  }
  transformProperties(content) {
    const properties = this.parseProperties(content);
    const transformed = {};
    for (const [key, value] of Object.entries(properties)) {
      transformed[key] = this.convertUnicodeToNative(value);
    }
    return transformed;
  }
  parseProperties(content) {
    const result = {};
    const lines = content.split(/\r?\n/);
    for (let line of lines) {
      line = line.trim();
      if (line && !line.startsWith('#')) {
        const [key, ...valueParts] = line.split('=');
        if (key && valueParts.length > 0) {
          const value = valueParts.join('=').trim();
          result[key.trim()] = value;
        }
      }
    }
    return result;
  }
  convertUnicodeToNative(str) {
    return str.replace(/\\u([0-9a-fA-F]{4})/g, (match, hex) => {
      try {
        return String.fromCharCode(parseInt(hex, 16));
      } catch (err) {
        return match;
      }
    });
  }
  getOutputFileName(relativePath) {
    const ext = path.extname(relativePath);
    const baseName = path.basename(relativePath, ext);
    const dirName = path.dirname(relativePath);
    let outputFileName = `${baseName}.${this.options.outputFormat}`;
    if (dirName && dirName !== '.') {
      outputFileName = path.join(dirName, outputFileName);
    }
    return path.join(this.options.outputDir, outputFileName);
  }
  getOutputContent(transformed) {
    if (this.options.outputFormat === 'json') {
      return JSON.stringify(transformed, null, 2);
    } else if (this.options.outputFormat === 'js') {
      return `module.exports = ${JSON.stringify(transformed, null, 2)};`;
    }
    throw new Error(`Unsupported output format: ${this.options.outputFormat}`);
  }
}
module.exports = I18nPropertiesPlugin;