"use strict";

const path = require('path');
const fs = require('fs');
const crypto = require('crypto');
const {
  walk
} = require('estree-walker');
const {
  RawSource
} = require('webpack-sources');
const parser = require('@babel/parser');
const generate = require('@babel/generator').default;
const pluginName = 'I18nKeyHashPlugin';
class I18nKeyHashPlugin {
  constructor(options = {}) {
    const {
      jsResourcePath
    } = options;
    if (!jsResourcePath) {
      throw new Error('jsResourcePath is required');
    }
    this.jsResourcePath = jsResourcePath;
    this.keyToHashMap = new Map();
    this.processedFiles = new WeakSet();
    this.preserveStringKeys = new Set(['Delete',
    // Common UI strings that shouldn't be hashed
    'Cancel', 'Save', 'Close']);
  }
  hashKey(key) {
    // Don't hash preserved strings
    if (this.preserveStringKeys.has(key)) {
      return key;
    }
    if (this.keyToHashMap.has(key)) {
      return this.keyToHashMap.get(key);
    }
    const hash = crypto.createHash('md5').update(key).digest('hex').substring(0, 7);
    this.keyToHashMap.set(key, hash);
    return hash;
  }
  loadI18nKeys() {
    try {
      const content = fs.readFileSync(this.jsResourcePath, 'utf8');
      const i18nObj = {};
      content.split(/\r?\n/).forEach(line => {
        const trimmedLine = line.trim();
        if (!trimmedLine || trimmedLine.startsWith('#')) return;
        const ind = trimmedLine.indexOf('=');
        if (ind <= 0) return;
        const key = trimmedLine.slice(0, ind).trim();
        const value = trimmedLine.slice(ind + 1).trim();
        if (key && value) {
          i18nObj[key] = value; // Pre-generate hash

          this.hashKey(key);
        }
      });
      return i18nObj;
    } catch (err) {
      return {};
    }
  }
  safeGenerateCode(ast) {
    try {
      const result = generate(ast, {
        retainLines: true,
        compact: false,
        concise: false,
        quotes: 'single',
        jsonCompatibleStrings: true
      });
      return result.code;
    } catch (err) {
      throw new Error(`Code generation failed: ${err.message}`);
    }
  }
  processSafeStringLiteral(node, hash) {
    // Handle template literals more carefully
    if (node.type === 'TemplateLiteral') {
      return; // Skip template literals for safety
    } // Handle regular string literals

    if ((node.type === 'StringLiteral' || node.type === 'Literal') && typeof node.value === 'string') {
      node.value = hash;
      node.raw = `'${hash}'`;
      if (node.extra) {
        node.extra.raw = `'${hash}'`;
        node.extra.rawValue = hash;
      }
    }
  }
  processAST(ast, i18nKeys) {
    let modified = false;
    try {
      walk(ast, {
        enter: node => {
          // Handle string literals in JSX attributes
          if ((node.type === 'StringLiteral' || node.type === 'Literal') && typeof node.value === 'string' && i18nKeys[node.value]) {
            const hash = this.hashKey(node.value);
            this.processSafeStringLiteral(node, hash);
            modified = true;
          } // Handle JSXText differently

          if (node.type === 'JSXText' && i18nKeys[node.value.trim()]) {
            const key = node.value.trim();
            const hash = this.hashKey(key);
            node.value = hash;
            modified = true;
          }
        }
      });
    } catch (err) {
      throw new Error(`AST processing failed: ${err.message}`);
    }
    return modified;
  }
  apply(compiler) {
    let i18nKeys;
    compiler.hooks.compilation.tap(pluginName, compilation => {
      if (!i18nKeys) {
        i18nKeys = this.loadI18nKeys();
      }
      compilation.hooks.succeedModule.tap(pluginName, module => {
        if (!module.resource || !/\.(js|jsx|ts|tsx)$/.test(module.resource) || this.processedFiles.has(module)) {
          return;
        }
        try {
          const source = module._source.source(); // Parse with more precise settings

          const ast = parser.parse(source, {
            sourceType: 'module',
            plugins: ['jsx', 'typescript', 'classProperties', 'decorators-legacy'],
            tokens: true,
            errorRecovery: true
          });
          if (this.processAST(ast, i18nKeys)) {
            const newCode = this.safeGenerateCode(ast);
            module._source = new RawSource(newCode); // Update module hash

            module.buildInfo.hash = crypto.createHash('md5').update(newCode).digest('hex');
          }
          this.processedFiles.add(module);
        } catch (err) {
          // Log error but don't fail the build
          compilation.warnings.push(new Error(`[${pluginName}] Processing ${module.resource} failed: ${err.message}`));
        }
      }); // Generate manifest after processing

      compilation.hooks.afterOptimizeModules.tap(pluginName, () => {
        const manifest = {
          hashMapping: Object.fromEntries(this.keyToHashMap),
          metadata: {
            totalKeys: this.keyToHashMap.size,
            timestamp: Date.now()
          }
        };
        compilation.assets['i18n-hash-manifest.json'] = new RawSource(JSON.stringify(manifest, null, 2));
      });
    });
  }
}
module.exports = I18nKeyHashPlugin;