"use strict";

const {
  RawSource
} = require('webpack-sources');
function generateManifests({
  compilation,
  allI18nKeys,
  moduleToKeysMap,
  cacheGroupToModulesMap,
  jsResourceI18nKeys
}) {
  const manifest = {
    cacheGroupToI18nMap: {},
    locales: {}
  };
  const metaManifest = {
    keyToCacheGroupMap: {}
  };
  const locales = Object.keys(allI18nKeys); // Map keys to cache groups

  const keyToCacheGroupMap = {};
  for (const [cacheGroupName, moduleResources] of cacheGroupToModulesMap.entries()) {
    const cacheGroupI18nKeys = new Set();
    for (const moduleResource of moduleResources) {
      const keys = moduleToKeysMap.get(moduleResource);
      if (keys) {
        keys.forEach(key => {
          if (jsResourceI18nKeys.hasOwnProperty(key)) {
            cacheGroupI18nKeys.add(key); // Map key to cache group for meta manifest

            if (!keyToCacheGroupMap[key]) {
              keyToCacheGroupMap[key] = new Set();
            }
            keyToCacheGroupMap[key].add(cacheGroupName);
          }
        });
      }
    } // Generate i18n files per cache group and locale

    for (const locale of locales) {
      const i18nData = {};
      const localeData = allI18nKeys[locale];
      for (const key of cacheGroupI18nKeys) {
        // Find the key in the locale data
        let found = false;
        for (const section of Object.values(localeData)) {
          if (section.hasOwnProperty(key)) {
            i18nData[key] = section[key];
            found = true;
            break;
          }
        }
        if (!found) {
          console.warn(`Key "${key}" not found in locale "${locale}"`);
        }
      } // Create the i18n chunk file

      const moduleSource = `window.loadI18nChunk(${JSON.stringify(i18nData, null, 2)});`;
      const filename = `i18n-chunks/${locale}/${cacheGroupName}.js`;
      compilation.emitAsset(filename, new RawSource(moduleSource)); // Update the main manifest

      if (!manifest.locales[locale]) {
        manifest.locales[locale] = {};
      }
      manifest.locales[locale][cacheGroupName] = filename;
    } // Update the cacheGroupToI18nMap

    manifest.cacheGroupToI18nMap[cacheGroupName] = cacheGroupName;
  } // Generate the meta manifest by converting Sets to Arrays

  for (const [key, cacheGroupsSet] of Object.entries(keyToCacheGroupMap)) {
    metaManifest.keyToCacheGroupMap[key] = Array.from(cacheGroupsSet);
  } // Emit the main manifest file

  const manifestSource = new RawSource(`window.i18n = window.i18n || {}; window.i18n.manifest = ${JSON.stringify(manifest, null, 2)};`);
  const manifestFilename = 'i18n-chunk/manifest.js';
  compilation.emitAsset(manifestFilename, manifestSource);
  console.log(`Emitted i18n manifest: ${manifestFilename}`); // Emit the meta manifest file

  const metaManifestSource = new RawSource(JSON.stringify(metaManifest, null, 2));
  const metaManifestFilename = 'i18n-chunk/metaManifest.json';
  compilation.emitAsset(metaManifestFilename, metaManifestSource);
  console.log(`Emitted i18n meta manifest: ${metaManifestFilename}`);
}
module.exports = {
  generateManifests
};