"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.VALIDATION = exports.RUNTIME_CONFIG = exports.RETRY_STRATEGY = exports.PERFORMANCE = exports.MANIFEST_FIELDS = exports.LOADING_MODES = exports.FILE_PATTERNS = exports.FILE_DEFAULTS = exports.FEATURES = exports.ERROR_MESSAGES = exports.CHUNK_SIZE_LIMITS = exports.CHUNK_HINTS = exports.CACHE_CONFIG = exports.ALL_I18N_KEYS = void 0;
// constants.js
// Chunk size limits
const CHUNK_SIZE_LIMITS = {
  // Maximum keys in a single chunk (prevent >60% keys in one chunk)
  MAX_CHUNK_SIZE: 200,
  // Minimum keys before considering chunk merge
  MIN_CHUNK_SIZE: 50,
  // Ideal chunk size for optimal loading
  IDEAL_CHUNK_SIZE: 100,
  // Maximum percentage of total keys in one chunk
  MAX_CHUNK_PERCENTAGE: 0.2,
  // 20%
  // Chunk size variance allowed before rebalancing
  CHUNK_SIZE_VARIANCE: 0.2 // 20% variance allowed
}; // Cache settings

exports.CHUNK_SIZE_LIMITS = CHUNK_SIZE_LIMITS;
const CACHE_CONFIG = {
  // Maximum number of chunks to cache
  MAX_CACHED_CHUNKS: 50,
  // Maximum age of cached chunks (ms)
  CACHE_TTL: 3600000,
  // 1 hour
  // Cache version for cache busting
  CACHE_VERSION: '1.0.0'
}; // File patterns

exports.CACHE_CONFIG = CACHE_CONFIG;
const FILE_PATTERNS = {
  // JavaScript file extensions to scan
  JS_EXTENSIONS: /\.(js|jsx|ts|tsx)$/,
  // i18n key patterns to match
  I18N_KEY_PATTERN: /^[a-zA-Z0-9_.-]+$/,
  // Comment patterns for i18n keys
  I18N_COMMENT_PATTERN: /^i18n:/
}; // Loading modes

exports.FILE_PATTERNS = FILE_PATTERNS;
const LOADING_MODES = {
  EAGER: 'eager',
  // Load chunks immediately
  LAZY: 'lazy',
  // Load chunks on demand
  PREFETCH: 'prefetch',
  // Load chunks after main content
  PRELOAD: 'preload' // Load chunks with high priority
}; // Default file names and paths

exports.LOADING_MODES = LOADING_MODES;
const FILE_DEFAULTS = {
  // Default manifest file name
  MANIFEST_FILE: 'i18n-manifest.json',
  // Fallback manifest file name
  FALLBACK_MANIFEST_FILE: 'i18n-manifest.main.json',
  // Stats file name
  STATS_FILE: 'i18n-stats.json',
  // Default chunk file pattern
  CHUNK_FILENAME: 'i18n-chunk/[locale]/[name].[chunkhash].i18n.js'
}; // Error messages

exports.FILE_DEFAULTS = FILE_DEFAULTS;
const ERROR_MESSAGES = {
  NO_LOCALES: 'No locales specified in plugin options',
  INVALID_KEY: 'Invalid i18n key format',
  CHUNK_OVERFLOW: 'Chunk size exceeds maximum limit',
  MANIFEST_ERROR: 'Failed to generate manifest',
  LOAD_ERROR: 'Failed to load i18n chunk',
  INVALID_CONFIG: 'Invalid plugin configuration'
}; // Feature flags for experimental features

exports.ERROR_MESSAGES = ERROR_MESSAGES;
const FEATURES = {
  ENABLE_KEY_MOVEMENT_TRACKING: true,
  ENABLE_CHUNK_STATS: true,
  ENABLE_SMART_LOADING: true,
  ENABLE_CACHE_OPTIMIZATION: true,
  ENABLE_DEBUG_LOGGING: false
}; // Performance thresholds

exports.FEATURES = FEATURES;
const PERFORMANCE = {
  // Maximum time to spend on chunk balancing (ms)
  MAX_BALANCE_TIME: 1000,
  // Maximum keys to process in one batch
  BATCH_SIZE: 1000,
  // Warning thresholds
  WARNINGS: {
    CHUNK_SIZE: 150,
    // Warn when chunk exceeds this size
    TOTAL_SIZE: 5000,
    // Warn when total keys exceed this
    LOAD_TIME: 500 // Warn when chunk load exceeds this (ms)
  }
}; // Runtime configurations that can be overridden

exports.PERFORMANCE = PERFORMANCE;
const RUNTIME_CONFIG = {
  // Default timeout for chunk loading (ms)
  CHUNK_TIMEOUT: 30000,
  // Number of retry attempts for failed loads
  MAX_RETRIES: 3,
  // Delay between retries (ms)
  RETRY_DELAY: 1000,
  // Enable verbose logging
  DEBUG_MODE: false
}; // Manifest metadata fields

exports.RUNTIME_CONFIG = RUNTIME_CONFIG;
const MANIFEST_FIELDS = {
  // Required fields in manifest
  REQUIRED: ['version', 'generated', 'chunks', 'locales'],
  // Optional metadata fields
  METADATA: ['description', 'environment', 'buildId', 'commitHash']
}; // Default retry strategy

exports.MANIFEST_FIELDS = MANIFEST_FIELDS;
const RETRY_STRATEGY = {
  // Initial delay (ms)
  initialDelay: 1000,
  // Maximum delay (ms)
  maxDelay: 10000,
  // Backoff factor
  factor: 2,
  // Maximum number of retries
  maxRetries: 3
}; // Chunk optimization hints

exports.RETRY_STRATEGY = RETRY_STRATEGY;
const CHUNK_HINTS = {
  // Threshold for splitting chunks
  SPLIT_THRESHOLD: 0.8,
  // 80% of max size
  // Threshold for merging chunks
  MERGE_THRESHOLD: 0.3,
  // 30% of max size
  // Preferred chunk size ranges
  RANGES: {
    SMALL: 50,
    MEDIUM: 100,
    LARGE: 150
  }
}; // Validation schemas (using simple pattern matching)

exports.CHUNK_HINTS = CHUNK_HINTS;
const VALIDATION = {
  KEY_PATTERN: /^[a-zA-Z0-9_.]+$/,
  LOCALE_PATTERN: /^[a-z]{2}(_[A-Z]{2})?$/,
  CHUNK_NAME_PATTERN: /^[a-zA-Z0-9-_]+$/
}; // Export all i18n keys for validation (this would be populated from your source)

exports.VALIDATION = VALIDATION;
const ALL_I18N_KEYS = new Set([
// Example keys - in real implementation, these would be loaded from your source
'common.ok', 'common.cancel', 'errors.notFound' // ... more keys
]);
exports.ALL_I18N_KEYS = ALL_I18N_KEYS;